require_relative "player"
require_relative "die"
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGameModule
class Game
  attr_reader :title
  def initialize(title)
    @title = title
    @players = []
  end

  def add_player(player)
    @players << player
  end
  def load_players(filename="players.csv")
    File.readlines(filename).each do |line|
      add_player(Player.from_csv(line))
    end
  end
  def save_high_scores(filename="high_scores.txt")
    File.open(filename, "w") do |file|
      file.puts("Knuckleheads High Scores:")
      @players.sort.each do |p|
        file.puts formatted_player(p)
      end
    end
  end
  def play(rounds)
    puts "There are #{@players.size} players in #{@title}: "

    display_players

    display_treasures(TreasureTrove::TREASURES)

    1.upto(rounds) do |r|
      puts "\nRound #{r}:"
      @players.each do |p|
        GameTurn.take_turn(p)
        puts p
      end
    end
  end

  def print_stats
    strongs = @players.select {|n| n.strong?}
    wimpies = @players.reject {|n| n.strong?}
    #puts strongs.size

    puts ""
    puts "Knuckleheads Statistics:"
    display_player_stats("strong", strongs)
    display_player_stats("wimpy", wimpies)

    puts ""
    puts "Knuckleheads High Scores:"
    display_players_score

    # puts ""
    # display_total_points

    puts ""
    display_total_points_per_player
  end

  private
  def formatted_player(player)
    "#{player.name.ljust(30, '.')} (#{player.score})"
  end
  def display_total_points_per_player
    @players.each do |p|
      puts ""
      puts "#{p.name} point totals:"
      p.each_found_treasure do |treasure|
        puts "#{treasure.points} total  #{treasure.name} points"
      end
      puts "#{p.points} grand total points"
    end
  end
  def display_total_points
    @players.each do |p|
      puts "#{p.name} point totals:"
      puts "#{p.points} grand total points"
      puts ""
    end
  end
  def display_players
    @players.each do |p|
      puts p
    end
  end
  def display_treasures(treasures)
    puts ""
    puts "number of treasures : #{treasures.size}"
    treasures.each do |t|
      puts "A #{t.name} is worth #{t.points} points"
    end
  end
  def display_player_stats(kind, players_array)
    puts ""
    puts "#{players_array.size} #{kind} players"
    players_array.each do |p|
      puts "#{p.name} (#{p.health})"
    end
  end
  def display_players_score
    # sorted_players =  @players.sort { |a,b| b.score <=> a.score }
    # sorted_players.each do |p|
    #   puts "#{p.name.ljust(30, '.')} (#{p.score})"
    # end
    @players.sort.each do |p|
     puts formatted_player(p)
   end
  end
end

end
